#! python3
import pandas as pd
import argparse
import matplotlib.pyplot as plt
from matplotlib import rcParams
import os
import numpy as np
import json
import pprint
import seaborn as sns

import matplotlib
matplotlib.use('Agg')


class TimeseriesData:
    timestamps = []
    operations = []
    latencies = []
    scaleOutAverage = None


write_base_path = "C:/git/omi-gitlab/mowgli-results/elasticity/overload-calibration/results/write-only/openstack"
write_data_files = [
    ["/cassandra/nodes-3_replication-3_write-one_threads-1/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-1/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-2/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-2/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-4/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-4/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-8/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-8/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-16/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-16/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-32/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-32/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-64/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-64/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-128/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-128/plots/data.json"]
]

read_base_path = "C:/git/omi-gitlab/mowgli-results/elasticity/overload-calibration/results/read-only/openstack"
read_data_files = [
    ["/cassandra/3-node_read-consistency-one_1-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_1-threads_read-only/plots/data.json"],
    ["/cassandra/3-node_read-consistency-one_2-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_2-threads_read-only/plots/data.json"],
    ["/cassandra/3-node_read-consistency-one_4-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_4-threads_read-only/plots/data.json"],
    ["/cassandra/3-node_read-consistency-one_8-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_8-threads_read-only/plots/data.json"],
    ["/cassandra/3-node_read-consistency-one_16-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_16-threads_read-only/plots/data.json"],
    ["/cassandra/3-node_read-consistency-one_32-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_32-threads_read-only/plots/data.json"],
    ["/cassandra/3-node_read-consistency-one_64-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_64-threads_read-only/plots/data.json"],
    ["/cassandra/3-node_read-consistency-one_128-threads_read-only/plots/data.json",
    "/couchbase/3-node_read-consistency-one_128-threads_read-only/plots/data.json"]
]

matplotlib.pyplot.autoscale()




def load():
    data = []
    count = 1
    #dirty hack from hell to get as well the read-only data.json files
    filecounter = 0
    for f_tupl in write_data_files:    
        path = write_base_path + f_tupl[0]
        filtered_json = {}
        with open(path, 'r') as file:
            raw_file_data = file.read()
            j = json.loads(raw_file_data)
            filtered_json['cassandra-write'] = j['metricsLoadPhase']['throughput']['avg']
            filtered_json['cassandra-write_std'] = j['metricsLoadPhase']['throughput']['std']

        path = write_base_path + f_tupl[1]
        with open(path, 'r') as file:
            raw_file_data = file.read()
            j = json.loads(raw_file_data)
            filtered_json['couchbase-write'] = j['metricsLoadPhase']['throughput']['avg']
            filtered_json['couchbase-write_std'] = j['metricsLoadPhase']['throughput']['std']
            filtered_json["YCSB client threads"] = count

        #read-only parts
        path = read_base_path + str(read_data_files[filecounter][0])
        with open(path, 'r') as file:
            raw_file_data = file.read()
            j = json.loads(raw_file_data)
            filtered_json['cassandra-read'] = j['metricsTransactionPhase']['throughput']['avg']
            filtered_json['cassandra-read_std'] = j['metricsTransactionPhase']['throughput']['std']

        path = read_base_path + str(read_data_files[filecounter][1])
        with open(path, 'r') as file:
            raw_file_data = file.read()
            j = json.loads(raw_file_data)
            filtered_json['couchbase-read'] = j['metricsTransactionPhase']['throughput']['avg']
            filtered_json['couchbase-read_std'] = j['metricsTransactionPhase']['throughput']['std']    

        data.append(filtered_json)
        count = count*2
        filecounter = filecounter + 1
    return data


def to_df(json):
    df = pd.DataFrame.from_records(json)
    df = df.set_index('YCSB client threads')
    print(df)
    return df

def plot(df):
    rcParams.update({'figure.autolayout': True})
    sns.set(style="darkgrid")
    sns.set_context("notebook", font_scale=1)
    sns.set_style({'font.family': 'serif'})
    #sns.set_palette("muted")
    sns.set_palette(['steelblue','royalblue','lightcoral','indianred'])

    bw_tp = df[['cassandra-write','cassandra-read','couchbase-write','couchbase-read']]
    bw_tp_std = df[['cassandra-write_std','cassandra-read_std','couchbase-write_std','couchbase-read_std']]
    bw_tp_std = bw_tp_std.rename(columns={"cassandra-write_std":"cassandra-write","cassandra-read_std":"cassandra-read","couchbase-write_std":"couchbase-write","couchbase-read_std":"couchbase-read"})

    ax = bw_tp.plot.bar(
        title="",
        yerr=bw_tp_std,
        rot=90,
        capsize=2,
        figsize=(7, 5)
    )
    ax.set_ylabel("avg. throughput in ops/s")
    ax.set_xlabel("workload intensities by YCSB client threads ")

    #plt.savefig("./test.png")
    plt.savefig("./barplot.pdf")


raw_data = load()
df = to_df(raw_data)
print(df)
plot(df)

