#! python3
import pandas as pd
import argparse
import matplotlib.pyplot as plt
from matplotlib import rcParams
import os
import numpy as np
import json
import pprint
import seaborn as sns

import matplotlib
matplotlib.use('Agg')


class TimeseriesData:
    timestamps = []
    operations = []
    latencies = []
    scaleOutAverage = None


base_path = "C:/git/omi-gitlab/mowgli-results/elasticity/overload-calibration/results/sensorstorage/openstack"
data_files = [
    ["/cassandra/nodes-3_replication-3_write-one_threads-1/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-1/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-2/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-2/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-4/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-4/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-8/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-8/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-16/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-16/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-32/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-32/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-64/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-64/plots/data.json"],
    ["/cassandra/nodes-3_replication-3_write-one_threads-128/plots/data.json",
     "/couchbase/nodes-3_replication-3_consistency-none_threads-128/plots/data.json"]
]

matplotlib.pyplot.autoscale()




def load():
    data = []
    count = 1
    for f_tupl in data_files:
        path = base_path + f_tupl[0]
        filtered_json = {}
        with open(path, 'r') as file:
            raw_file_data = file.read()
            j = json.loads(raw_file_data)
            filtered_json['cassandra'] = j['metricsLoadPhase']['throughput']['avg']
            filtered_json['cassandra_std'] = j['metricsLoadPhase']['throughput']['std']

        path = base_path + f_tupl[1]
        with open(path, 'r') as file:
            raw_file_data = file.read()
            j = json.loads(raw_file_data)
            filtered_json['couchbase'] = j['metricsLoadPhase']['throughput']['avg']
            filtered_json['couchbase_std'] = j['metricsLoadPhase']['throughput']['std']
            filtered_json["YCSB client threads"] = count
        data.append(filtered_json)
        count = count*2
    return data


def to_df(json):
    df = pd.DataFrame.from_records(json)
    df = df.set_index('YCSB client threads')
    return df

def plot(df):
    rcParams.update({'figure.autolayout': True})
    sns.set(style="darkgrid")
    sns.set_context("notebook", font_scale=1)
    sns.set_style({'font.family': 'serif'})
    sns.set_palette("muted")

    bw_tp = df[['cassandra','couchbase']]
    bw_tp_std = df[['cassandra_std','couchbase_std']]
    bw_tp_std = bw_tp_std.rename(columns={"cassandra_std":"cassandra","couchbase_std":"couchbase"})

    ax = bw_tp.plot.bar(
        title="avg. write throughput - 3 node Cassandra/Couchbase cluster",
        yerr=bw_tp_std,
        rot=90,
        capsize=2,
        figsize=(7, 5)
    )
    ax.set_ylabel("Ops/s")

    plt.savefig("./test.png")
    plt.savefig("./test.pdf")


raw_data = load()
df = to_df(raw_data)
print(df)
plot(df)

