package de.uulm.omi.dbms.evlautor.server.api;

import de.uulm.omi.cloudiator.dbms.evaluator.model.*;
import de.uulm.omi.dbms.evlautor.server.api.ScenarioApiService;
import de.uulm.omi.dbms.evlautor.server.api.factories.ScenarioApiServiceFactory;

import io.swagger.annotations.ApiParam;
import io.swagger.jaxrs.*;

import de.uulm.omi.cloudiator.dbms.evaluator.model.AvailabilityYCSBMultiPhase;
import de.uulm.omi.cloudiator.dbms.evaluator.model.AvailabilityYCSBWrite;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ElasticityYCSBMultiPhase;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ElasticityYCSBWrite;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Error;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationStatus;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationSubmissionError;
import de.uulm.omi.cloudiator.dbms.evaluator.model.NoiseyNeighbourScenario;
import de.uulm.omi.cloudiator.dbms.evaluator.model.PerformanceTPCC;
import de.uulm.omi.cloudiator.dbms.evaluator.model.PerformanceTSBS;
import de.uulm.omi.cloudiator.dbms.evaluator.model.PerformanceYCSBMultiPhase;
import de.uulm.omi.cloudiator.dbms.evaluator.model.PerformanceYCSBWrite;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ScalabilityTPCC;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ScalabilityTSBS;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ScalabilityYCSBMultiPhase;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ScalabilityYCSBWrite;

import java.util.Map;
import java.util.List;
import de.uulm.omi.dbms.evlautor.server.api.NotFoundException;

import java.io.InputStream;

import org.glassfish.jersey.media.multipart.FormDataContentDisposition;
import org.glassfish.jersey.media.multipart.FormDataParam;

import javax.servlet.ServletConfig;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.*;
import javax.validation.constraints.*;

@Path("/scenario")

@Produces({ "application/json" })
@io.swagger.annotations.Api(description = "the scenario API")
@javax.annotation.Generated(value = "io.swagger.codegen.languages.JavaJerseyServerCodegen", date = "2020-05-19T16:46:41.854+02:00")
public class ScenarioApi  {
   private final ScenarioApiService delegate;

   public ScenarioApi(@Context ServletConfig servletContext) {
      ScenarioApiService delegate = null;

      if (servletContext != null) {
         String implClass = servletContext.getInitParameter("ScenarioApi.implementation");
         if (implClass != null && !"".equals(implClass.trim())) {
            try {
               delegate = (ScenarioApiService) Class.forName(implClass).newInstance();
            } catch (Exception e) {
               throw new RuntimeException(e);
            }
         } 
      }

      if (delegate == null) {
         delegate = ScenarioApiServiceFactory.getScenarioApi();
      }

      this.delegate = delegate;
   }

    @POST
    @Path("/availabilityMultiPhase")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Multi-Phase Availability", notes = "Deploy a DDBMS on IaaS, issue a CRUD workload, and inject failures during the transaction phase | evaluate performance in case of node failures ", response = EvaluationStatus.class, tags={ "availability", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioAvailabilityMultiPhasePost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Availability-YCSB-MULTI-PHASE") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODBSHARDED, MONGODBREPLICATED, COCKROACHDB, RIAK, ELASTICSEARCH") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorageGibbon scenario specification including the failure spec" ,required=true) AvailabilityYCSBMultiPhase scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioAvailabilityMultiPhasePost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/availability")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Load Availability", notes = "Deploy a DDBMS on IaaS, issue a write-heavy workload, evaluate write performance for different VM and DDBMS configurations | in case of node failures", response = EvaluationStatus.class, tags={ "availability", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioAvailabilityPost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Availability-YCSB-Write") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODBSHARDED, MONGODBREPLICATED, COCKROACHDB, RIAK, ELASTICSEARCH") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorageGibbon scenario specification including the failure spec" ,required=true) AvailabilityYCSBWrite scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioAvailabilityPost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/DockerPerformanceMultiPhase")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Multi-Phase Performance (single node) for custom Docker configurations", notes = "Deploy a containerized single DBMS on IaaS, issue a write-heavy workload and a custom CRUD worklaod, evaluate performance for different VM and DBMS configurations ", response = EvaluationStatus.class, tags={ "docker", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioDockerPerformanceMultiPhasePost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Docker-Performance-YCSB-MULTI-PHASE") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="MONGODBDOCKER") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) PerformanceYCSBMultiPhase scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioDockerPerformanceMultiPhasePost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/ElasticityMultiPhase")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Multi-Phase Elasticity", notes = "Evalaute the read/update/scan throughput of DDBMS with different YCSB configurations and while scaling the DBMS at runtime", response = EvaluationStatus.class, tags={ "elasticity", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioElasticityMultiPhasePost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Elasticity-YCSB-MULTI-PHASE") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODBSHARDED, MONGODBREPLICATED, COCKROACHDB, RIAK, ELASTICSEARCH") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a plain string to describe relevant dbms configuration | e.g. 2_node-persist_to_1. Will be used to create to 3rd level folders for each configuration",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the elasticity scenario specification including the deployment, workload and elasticity specs" ,required=true) ElasticityYCSBMultiPhase elasticitySpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioElasticityMultiPhasePost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,elasticitySpec,securityContext);
    }
    @POST
    @Path("/Elasticity")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Load Elasticity", notes = "Evalaute the write throughput of DDBMS with different YCSB configurations and while scaling the DBMS at runtime", response = EvaluationStatus.class, tags={ "elasticity", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioElasticityPost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Elasticity-YCSB-Write") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODBSHARDED, MONGODBREPLICATED, COCKROACHDB, RIAK, ELASTICSEARCH") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a plain string to describe relevant dbms configuration | e.g. 2_node-persist_to_1. Will be used to create to 3rd level folders for each configuration",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the elasticity scenario specification including the deployment, workload and elasticity specs" ,required=true) ElasticityYCSBWrite elasticitySpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioElasticityPost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,elasticitySpec,securityContext);
    }
    @POST
    @Path("/noiseyNeighbour")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "TODO", notes = "Deploy a DDBMS on IaaS, issue DBMS workloads and a HPC in parallel for different VM configs  ", response = EvaluationStatus.class, tags={ "resource interference", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioNoiseyNeighbourPost(@ApiParam(value = "the name of the new evaluation scenario",required=true) @QueryParam("scenarioName") String scenarioName
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) NoiseyNeighbourScenario scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioNoiseyNeighbourPost(scenarioName,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/performance/TPCC")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "TPC-C", notes = "Evalaute the TPC-C performance of a CockroachDB instance with different TPC-C configurations", response = EvaluationStatus.class, tags={ "performance", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already an evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioPerformanceTPCCPost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Scalability-TPCC") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="COCKROACHDB") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a plain string to describe relevant dbms configuration | e.g. 1_node. Will be used to create to 3rd level folders for each configuration",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) PerformanceTPCC scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioPerformanceTPCCPost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/performance/TSBS")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "TSBS", notes = "Deploy a single node DBMS on IaaS, issue a time-series workload, evaluate read and write performance for different VM and DBMS configurations ", response = EvaluationStatus.class, tags={ "performance", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioPerformanceTSBSPost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Performance-TSBS") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, MONGODB, INFLUX, POSTGRESQL, TIMESCALEDB") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) PerformanceTSBS scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioPerformanceTSBSPost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/performance/YCSBLoad")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Load", notes = "Deploy a single node DDBMS on IaaS, issue a write-heavy workload, evaluate write performance for different VM and DDBMS configurations    ", response = EvaluationStatus.class, tags={ "performance", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioPerformanceYCSBLoadPost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Performance-YCSB-WRITE") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODB, COCKROACHDB, RIAK, ELASTICSEARCH, POSTGRESQL") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) PerformanceYCSBWrite scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioPerformanceYCSBLoadPost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/performance/YCSBMultiPhase")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Multi-Phase", notes = "Deploy a single DBMS on IaaS, issue a write-heavy workload and a custom CRUD worklaod, evaluate performance for different VM and DBMS configurations ", response = EvaluationStatus.class, tags={ "performance", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioPerformanceYCSBMultiPhasePost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Performance-YCSB-MULTI-PHASE") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODB, COCKROACHDB, RIAK, ELASTICSEARCH, POSTGRESQL") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) PerformanceYCSBMultiPhase scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioPerformanceYCSBMultiPhasePost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/scalability/TPCC")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "TPC-C", notes = "Evalaute the TPC-C performance of the CockroachDB cluster with different TPC-C configurations", response = EvaluationStatus.class, tags={ "scalability", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already an evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioScalabilityTPCCPost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Scalability-TPCC") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="COCKROACHDB") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a plain string to describe relevant dbms configuration | e.g. 2_node-persist_to_1. Will be used to create to 3rd level folders for each configuration",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) ScalabilityTPCC scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioScalabilityTPCCPost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/scalability/TSBS")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "TSBS", notes = "Deploy a DBMS cluster on IaaS, issue a time-series workload, evaluate the performance and scalability for different VM and DBMS configurations ", response = EvaluationStatus.class, tags={ "scalability", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioScalabilityTSBSPost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Scalability-TSBS") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, MONGODB, INFLUX") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) ScalabilityTSBS scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioScalabilityTSBSPost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/scalability/YCSBLoadPhase")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Load", notes = "Deploy a DDBMS on IaaS, issue a write-heavy workload, evaluate write performance for different VM and DDBMS configurations", response = EvaluationStatus.class, tags={ "scalability", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioScalabilityYCSBLoadPhasePost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Scalability-YCSB-Write") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODBSHARDED, MONGODBREPLICATED, COCKROACHDB, RIAK, ELASTICSEARCH") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) ScalabilityYCSBWrite scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioScalabilityYCSBLoadPhasePost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
    @POST
    @Path("/scalability/YCSBMultiPhase")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "YCSB Multi-Phase", notes = "Deploy a DDBMS on IaaS, issue a write-heavy workload and a custom CRUD worklaod, evaluate performance for different VM and DDBMS configurations    ", response = EvaluationStatus.class, tags={ "scalability", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = EvaluationStatus.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already ae evaluation running", response = EvaluationSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response scenarioScalabilityYCSBMultiPhasePost(@ApiParam(value = "the scenario type identifier which will be used to create | the root folder for the evaluation results",required=true, allowableValues="Scalability-YCSB-MULTI-PHASE") @QueryParam("scenarioType") String scenarioType
,@ApiParam(value = "the cloud type identifier which will be used to create | the 2nd level folder for the evaluation results",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("cloud") String cloud
,@ApiParam(value = "the DBMS type identifier which will be used to create | the 3nd level folder for the evaluation results",required=true, allowableValues="CASSANDRA, COUCHBASE, MONGODBSHARDED, MONGODBREPLICATED, COCKROACHDB, RIAK, ELASTICSEARCH") @QueryParam("dbmsType") String dbmsType
,@ApiParam(value = "a custom string to describe relevant dbms configuration | for the this evaluation scenario, will be used for the 4th level of the evaluation results",required=true) @QueryParam("dbmsConfiguration") String dbmsConfiguration
,@ApiParam(value = "the number of interative evaluation runs",required=true) @QueryParam("runs") Integer runs
,@ApiParam(value = "delete all resources after each run",required=true) @QueryParam("cleanUp") Boolean cleanUp
,@ApiParam(value = "the sensorStorage scenario specification" ,required=true) ScalabilityYCSBMultiPhase scenarioSpec
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.scenarioScalabilityYCSBMultiPhasePost(scenarioType,cloud,dbmsType,dbmsConfiguration,runs,cleanUp,scenarioSpec,securityContext);
    }
}
