package de.uulm.omi.evaluation.scenario;

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.ParallelFlow.Builder.aNewParallelFlow;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.AvailabilityYCSBMultiPhase;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchYcsbResult;
import de.uulm.omi.evaluation.task.GibbonV2;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.InitRun;
import de.uulm.omi.evaluation.task.LogTask;
import de.uulm.omi.evaluation.task.Plotting;
import de.uulm.omi.evaluation.task.Plotting.PlottingTask;
import de.uulm.omi.evaluation.task.RemoteCommand;
import de.uulm.omi.evaluation.task.TaskState;
import de.uulm.omi.evaluation.task.TaskType;
import de.uulm.omi.evaluation.task.YcsbWorkload;
import de.uulm.omi.utils.YcsbWorkloadType;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;

/**
 * Created by Daniel Seybold on 27.09.2018.
 */
public class AvailabilityYCSBMultiPhaseScenario extends EvaluationScenario {
  private final AvailabilityYCSBMultiPhase availabilityYCSBMultiPhase;
  private final boolean cleanUp;
  private final ClusterContext dbmsClusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final int overallRuns;

  public AvailabilityYCSBMultiPhaseScenario(String scenarioType, String cloud, String dbmsType, String dbmsConfig, Integer runs,
      AvailabilityYCSBMultiPhase scenarioSpec, boolean cleanUp) {
    super(scenarioType,cloud, dbmsType , dbmsConfig, runs);

    this.cleanUp = cleanUp;
    this.availabilityYCSBMultiPhase = scenarioSpec;
    this.dbmsClusterContext = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    overallRuns = runs;
  }

  @Override
  public WorkReport execute() {

    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    InitRun initRun = new InitRun(this.evaluationContext);

    DeployDB deployDB = new DeployDB(this.evaluationContext,
        this.availabilityYCSBMultiPhase.getDbmsCluster(),
        this.availabilityYCSBMultiPhase.getDbmsNetwork().getType().name(), this.dbmsClusterContext);

    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext, this.dbmsClusterContext);

    CheckWorkload checkWorkload = new CheckWorkload(this.evaluationContext, this.workloadPhaseContext,this.availabilityYCSBMultiPhase
        .getWorkloadInstances());




    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    YcsbWorkload ycsbWorkloadLoad = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.availabilityYCSBMultiPhase.getWorkloadLoad(), this.availabilityYCSBMultiPhase.getWorkloadNetwork(), this.workloadPhaseContext,
        YcsbWorkloadType.LOAD,this.availabilityYCSBMultiPhase.getWorkloadInstances());

    FetchYcsbResult fetchYcsbResultLoad = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.LOAD,this.availabilityYCSBMultiPhase
        .getWorkloadInstances());

    YcsbWorkload ycsbWorkloadTransaction = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.availabilityYCSBMultiPhase.getWorkloadTransaction(), this.availabilityYCSBMultiPhase.getWorkloadNetwork(), this.workloadPhaseContext, YcsbWorkloadType.TRANSACTION,this.availabilityYCSBMultiPhase
        .getWorkloadInstances());

    FetchYcsbResult fetchYcsbResultTransaction = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.TRANSACTION,this.availabilityYCSBMultiPhase
        .getWorkloadInstances());

    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, dbmsClusterContext);


    /**
     * Gibbon specification
     */
    /**
     * specify a delay before injecting a failure via Gibbon
     */
    //Delay gibbonDelay = new Delay(evaluationContext, this.availabilityYCSBMultiPhase.getGibbonSpec().getGibbonDelay());

    /**
     * create the Gibbon task to inject node failures
     */
    //Gibbon gibbon = new Gibbon(evaluationContext, this.dbmsClusterContext, this.availabilityYCSBMultiPhase
    //    .getGibbonSpec());
    /**
     * specify a delay before triggering the recovery delay
     */
    //Delay recoveryDelay = new Delay(evaluationContext, this.availabilityYCSBMultiPhase.getGibbonSpec().getRecoveryDelay());

    /**
     * trigger the recovery action, i.e. scale out the cluster with (currently) 1 node
     */
    //Recovery recovery = new Recovery(this.evaluationContext, this.dbmsClusterContext, this.availabilityYCSBMultiPhase
    //    .getGibbonSpec().isFailureRecovery(), ScalingTypeEnum.OUT);

    GibbonV2 gibbonV2 = new GibbonV2(this.evaluationContext,this.dbmsClusterContext,this.availabilityYCSBMultiPhase.getGibbonSpec());


    RemoteCommand remoteCommand = new RemoteCommand(this.evaluationContext,this.dbmsClusterContext,this.availabilityYCSBMultiPhase.getDbmsCluster().getType().name().toLowerCase());


    /**
     * Plot the timeseries of the current run
     */
    Plotting mergeYCSB = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.MERGE_YCSB_ALL);

    Plotting evaluationTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.EVALUATION_METRICS);

    Plotting systemTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.SYSTEM_METRICS);

    Plotting workloadTimeseries = new Plotting(this.evaluationContext,
        this.dbmsClusterContext, PlottingTask.WORKLOAD_METRICS, this.availabilityYCSBMultiPhase.getWorkloadInstances());

    Plotting plotAvailabilityPhases = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.AVAILABILITY_PHASES, "transaction");


    SequentialFlow gibbonFlow = aNewSequentialFlow()
        .execute(initRun)
        .execute(deployDB)
        .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.STARTED))
        .execute(checkDeployment)
        .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.FINISHED))
        .execute(ycsbWorkloadLoad)
        .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.STARTED))
        .execute(checkWorkload)
        .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.FINISHED))
        .execute(fetchYcsbResultLoad)
        .execute(aNewParallelFlow()
                        .execute(
                            aNewSequentialFlow()
                                .execute(ycsbWorkloadTransaction)
                                .execute(new LogTask(evaluationContext, TaskType.YCSB_TRANSACTION, TaskState.STARTED))
                                .execute(checkWorkload)
                                .execute(new LogTask(evaluationContext, TaskType.YCSB_TRANSACTION, TaskState.FINISHED))
                                .build(),
                            aNewSequentialFlow()
                                .execute(gibbonV2)
                                //.execute(new LogTask(evaluationContext, TaskType.GIBBON, TaskState.STARTED))
                                //.execute(gibbon)
                                //.execute(new LogTask(evaluationContext, TaskType.GIBBON, TaskState.FINISHED))
                                //.execute(recoveryDelay)
                                //.execute(recovery)
                                .build()
                        )
            .build()
        )
        .execute(fetchYcsbResultTransaction)
        .execute(remoteCommand)
        .execute(mergeYCSB)
        .execute(evaluationTimeseries)
        .execute(workloadTimeseries)
        .execute(systemTimeseries)
        .execute(plotAvailabilityPhases)
        .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.STARTED))
        .execute(cleanUpResources)
        .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.FINISHED))
        .execute(cleanUpRun)
        .build();




    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    gibbonFlow
                )
                .times(this.overallRuns)
                .build()
        )
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);

    return workReport;


  }
}
