package de.uulm.omi.evaluation.scenario;

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.ParallelFlow.Builder.aNewParallelFlow;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ElasticityYCSBMultiPhase;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchYcsbResult;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.InitRun;
import de.uulm.omi.evaluation.task.Kaa;
import de.uulm.omi.evaluation.task.LogTask;
import de.uulm.omi.evaluation.task.Plotting;
import de.uulm.omi.evaluation.task.Plotting.PlottingTask;
import de.uulm.omi.evaluation.task.TaskState;
import de.uulm.omi.evaluation.task.TaskType;
import de.uulm.omi.evaluation.task.YcsbWorkload;
import de.uulm.omi.utils.YcsbWorkloadType;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;

/**
 * Created by Daniel Seybold on 15.10.2018.
 */
public class ElasticityYCSBMultiPhaseScenario extends EvaluationScenario {
  private final ElasticityYCSBMultiPhase elasticityYCSBMultiPhase;
  private final boolean cleanUp;
  private final ClusterContext dbmsClusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final int overallRuns;

  private int elasticityCounter = 0;

  public ElasticityYCSBMultiPhaseScenario(String scenarioType, String cloud, String dbmsType,  String dbmsConfig, Integer runs,
      ElasticityYCSBMultiPhase elasticityYCSBMultiPhase,
      boolean cleanUp) {
    super(scenarioType,cloud,  dbmsType,  dbmsConfig, runs);

    //TODO: integrate workloadApiEndpoints for multi-phase workloads

    this.cleanUp = cleanUp;
    this.elasticityYCSBMultiPhase = elasticityYCSBMultiPhase;
    this.dbmsClusterContext = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    this.overallRuns = runs;
  }

  @Override
  public WorkReport execute() {

    /**
     **************************** Deployment *********************************************
     */

    /**
     * init general result folders and counters
     */
    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    /**
     * init evaluation result folder for each evaluation run
     */
    InitRun initRun = new InitRun(this.evaluationContext);

    /**
     * deploy the DBMS cluster
     */
    DeployDB deployDB = new DeployDB(this.evaluationContext, this.elasticityYCSBMultiPhase.getDbmsCluster(),
        this.elasticityYCSBMultiPhase.getDbmsNetwork().getType().name(), this.dbmsClusterContext);


    /**
     * check the deployment process
     */
    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext,
        this.dbmsClusterContext);

    /**
     **************************** Workload *********************************************
     */

    /**
     * start the YCSB load workload
     */
    YcsbWorkload ycsbWorkload = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.elasticityYCSBMultiPhase.getWorkloadLoad(), this.elasticityYCSBMultiPhase.getWorkloadNetwork(), this.workloadPhaseContext,
        YcsbWorkloadType.LOAD,this.elasticityYCSBMultiPhase.getWorkloadInstances());

    /**
     * check the state of the running workload
     */
    CheckWorkload checkWorkload = new CheckWorkload(this.evaluationContext, this.workloadPhaseContext,this.elasticityYCSBMultiPhase
        .getWorkloadInstances());


    YcsbWorkload ycsbWorkloadTransaction = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.elasticityYCSBMultiPhase.getWorkloadTransaction(), this.elasticityYCSBMultiPhase.getWorkloadNetwork(), this.workloadPhaseContext,
        YcsbWorkloadType.TRANSACTION,this.elasticityYCSBMultiPhase.getWorkloadInstances());

    FetchYcsbResult fetchYcsbResultTransaction = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.TRANSACTION,this.elasticityYCSBMultiPhase
        .getWorkloadInstances());

    /**
     **************************** Elasticity *********************************************
     * all elasticity task are specified directly within the flows as multiple steps are executed
     */



    /**
     **************************** Plotting *********************************************
     */

    /**
     * Fetch the workload results for the current run and store to disk
     */
    FetchYcsbResult fetchYcsbResult = new FetchYcsbResult(this.evaluationContext,this.workloadPhaseContext, YcsbWorkloadType.LOAD,this.elasticityYCSBMultiPhase
        .getWorkloadInstances());

    /**
     * CleanUpResources all resources of the current run
     */
    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    /**
     * Plot the timeseries of the current run
     */
    Plotting mergeYCSB = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.MERGE_YCSB_ALL);

    Plotting evaluationTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.EVALUATION_METRICS);

    Plotting systemTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.SYSTEM_METRICS);

    Plotting workloadTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.WORKLOAD_METRICS, this.elasticityYCSBMultiPhase
        .getWorkloadInstances());

    Plotting elasticityPhases = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.ELASTICITY_PHASES, this.elasticityYCSBMultiPhase
        .getElasticitySpec().get(0).getScalingDelay().intValue());


    /**
     * Increase the run counter
     */
    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, this.dbmsClusterContext);

    /**
     * Build the workflow with the created tasks
     */
    SequentialFlow runFlow;


    runFlow = aNewSequentialFlow()
        .execute(initRun)
        .execute(deployDB)
        .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.STARTED))
        .execute(checkDeployment)
        .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.FINISHED))
        .execute(ycsbWorkload)
        .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.STARTED))
        .execute(checkWorkload)
        .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.FINISHED))
        .execute(
            aNewParallelFlow()
                .execute(
                    aNewSequentialFlow()
                        .execute(ycsbWorkloadTransaction)
                        .execute(new LogTask(evaluationContext, TaskType.YCSB_TRANSACTION, TaskState.STARTED))
                        .execute(checkWorkload)
                        .execute(new LogTask(evaluationContext, TaskType.YCSB_TRANSACTION, TaskState.FINISHED))
                        .build(),
                    //aNewRepeatFlow()
                      //  .repeat(
                            aNewSequentialFlow()
                                .execute(new Kaa(evaluationContext, this.dbmsClusterContext, this.elasticityYCSBMultiPhase
                                    .getElasticitySpec()))
                                //.execute(new IncreaseCounter(evaluationContext, this.elasticityCounter))
                                .build()
                        //)
                        //.times(this.overallRuns)
                        //.build()
                )
                .build()
        )
        .execute(fetchYcsbResult)
        .execute(fetchYcsbResultTransaction)
        .execute(mergeYCSB)
        .execute(evaluationTimeseries)
        .execute(workloadTimeseries)
        .execute(systemTimeseries)
        .execute(elasticityPhases)
        .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.STARTED))
        .execute(cleanUpResources)
        .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.FINISHED))
        .execute(cleanUpRun)
        .build();


    /**
     * execute the specified workflows
     */
    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    runFlow
                )
                .times(this.overallRuns)
                .build()
        )
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);

    return workReport;

  }
}
