package de.uulm.omi.evaluation.scenario;

/**
 * Created by Daniel Seybold on 18.05.2018.
 */

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.PerformanceYCSBWrite;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchYcsbResult;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.InitRun;
import de.uulm.omi.evaluation.task.LogTask;
import de.uulm.omi.evaluation.task.Plotting;
import de.uulm.omi.evaluation.task.Plotting.PlottingTask;
import de.uulm.omi.evaluation.task.TaskState;
import de.uulm.omi.evaluation.task.TaskType;
import de.uulm.omi.evaluation.task.YcsbWorkload;
import de.uulm.omi.utils.YcsbWorkloadType;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;

/**
 * Created by Daniel Seybold on 09.05.2018.
 */
public class PerformanceYCSBWriteScenario extends EvaluationScenario {

  private final PerformanceYCSBWrite performanceYCSBWrite;
  private final ClusterContext dbmsClusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final int overallRuns;

  private final boolean cleanUp;

  public PerformanceYCSBWriteScenario(String scenarioType,String cloudType, String dbmsType, String dbmsConfiguraion, Integer runs,
      PerformanceYCSBWrite scenarioSpec, boolean cleanUp) {
    super(scenarioType,cloudType,  dbmsType, dbmsConfiguraion, runs);

    this.performanceYCSBWrite = scenarioSpec;
    this.dbmsClusterContext = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    this.overallRuns = runs.intValue();
    this.cleanUp = cleanUp;
  }

  @Override
  public WorkReport execute() {

    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    InitRun initRun = new InitRun(this.evaluationContext);

    DeployDB deployDB = new DeployDB(this.evaluationContext,
        this.performanceYCSBWrite.getDbmsSingle(), this.performanceYCSBWrite.getDbmsNetwork().getType().name(), this.dbmsClusterContext);

    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext, this.dbmsClusterContext);

    CheckWorkload checkWorkload = new CheckWorkload(this.evaluationContext, this.workloadPhaseContext,
        performanceYCSBWrite.getWorkloadInstances());

    FetchYcsbResult fetchYcsbResult = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.LOAD, performanceYCSBWrite
        .getWorkloadInstances());


    YcsbWorkload ycsbWorkload = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.performanceYCSBWrite.getWorkload(), this.performanceYCSBWrite.getWorkloadNetwork(), this.workloadPhaseContext,
        YcsbWorkloadType.LOAD, performanceYCSBWrite.getWorkloadInstances());

    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, this.dbmsClusterContext);

    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    /**
     * Plot the timeseries of the current run
     */
    Plotting mergeYCSB = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.MERGE_YCSB_LOAD);

    Plotting evaluationTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.EVALUATION_METRICS);

    Plotting systemTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.SYSTEM_METRICS);

    Plotting workloadTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.WORKLOAD_METRICS, performanceYCSBWrite
        .getWorkloadInstances());

    Plotting aggregateEvaluationMetrics = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.AGGREGATE_EVALUATION_METRICS);



    SequentialFlow runFlow;

    runFlow = aNewSequentialFlow()
        .execute(initRun)
        .execute(deployDB)
        .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.STARTED))
        .execute(checkDeployment)
        .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.FINISHED))
        .execute(ycsbWorkload)
        .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.STARTED))
        .execute(checkWorkload)
        .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.FINISHED))
        .execute(fetchYcsbResult)
        .execute(mergeYCSB)
        .execute(evaluationTimeseries)
        .execute(workloadTimeseries)
        .execute(systemTimeseries)
        .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.STARTED))
        .execute(cleanUpResources)
        .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.FINISHED))
        .execute(cleanUpRun)
        .build();

    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    runFlow
                )
                .times(this.overallRuns)
                .build()
        )
        .execute(aggregateEvaluationMetrics)
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);

    return workReport;


  }
}
