package de.uulm.omi.evaluation.scenario;

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ScalabilityTSBS;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchTSBSResult;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.InitRun;
import de.uulm.omi.evaluation.task.Plotting;
import de.uulm.omi.evaluation.task.Plotting.PlottingTask;
import de.uulm.omi.evaluation.task.TSBSWorkload;
import de.uulm.omi.evaluation.task.TSBSWorkload.TSBS_PHASE;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;

/**
 * Created by Daniel Seybold on 19.05.2020.
 */
public class ScalabilityTSBSScenario extends EvaluationScenario {

  private final ScalabilityTSBS scalabilityTSBS;
  private final ClusterContext dbmsClusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final int overallRuns;

  private final boolean cleanUp;

  public ScalabilityTSBSScenario(String scenarioType,String cloudType, String dbmsType, String dbmsConfiguraion, Integer runs,
      ScalabilityTSBS scenarioSpec, boolean cleanUp) {
    super(scenarioType,cloudType,  dbmsType, dbmsConfiguraion, runs);

    this.scalabilityTSBS = scenarioSpec;
    this.dbmsClusterContext = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    this.overallRuns = runs.intValue();
    this.cleanUp = cleanUp;
  }

  @Override
  public WorkReport execute() {

    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    InitRun initRun = new InitRun(this.evaluationContext);

    DeployDB deployDB = new DeployDB(this.evaluationContext,
        this.scalabilityTSBS.getDbmsCluster(), this.scalabilityTSBS.getDbmsNetwork().getType().name(), this.dbmsClusterContext);

    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext, this.dbmsClusterContext);



    TSBSWorkload generateTSBSData = new TSBSWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.scalabilityTSBS.getWorkload(), this.scalabilityTSBS.getWorkloadNetwork(), this.workloadPhaseContext,
        scalabilityTSBS.getWorkloadInstances(), TSBS_PHASE.GENERATE_DATA);

    TSBSWorkload loadTSBSData = new TSBSWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.scalabilityTSBS.getWorkload(), this.scalabilityTSBS.getWorkloadNetwork(), this.workloadPhaseContext,
        scalabilityTSBS.getWorkloadInstances(), TSBS_PHASE.LOAD_DATA);

    TSBSWorkload generateTSBSQueires = new TSBSWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.scalabilityTSBS.getWorkload(), this.scalabilityTSBS.getWorkloadNetwork(), this.workloadPhaseContext,
        scalabilityTSBS.getWorkloadInstances(), TSBS_PHASE.GENERATE_QUERIES);

    TSBSWorkload executeTSBSQueires = new TSBSWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.scalabilityTSBS.getWorkload(), this.scalabilityTSBS.getWorkloadNetwork(), this.workloadPhaseContext,
        scalabilityTSBS.getWorkloadInstances(), TSBS_PHASE.EXECUTE_QUERIES);


    CheckWorkload checkWorkload = new CheckWorkload(this.evaluationContext, this.workloadPhaseContext,
        scalabilityTSBS.getWorkloadInstances());

    FetchTSBSResult fetchTSBSResultLoad = new FetchTSBSResult(this.evaluationContext, this.workloadPhaseContext, scalabilityTSBS.getWorkloadInstances(), "load");

    FetchTSBSResult fetchTSBSResultQuery = new FetchTSBSResult(this.evaluationContext, this.workloadPhaseContext, scalabilityTSBS.getWorkloadInstances(), "query");


    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, this.dbmsClusterContext);

    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    /**
     * Plot the timeseries of the current run
     */
    Plotting systemTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.SYSTEM_METRICS);


    SequentialFlow runFlow;

    runFlow = aNewSequentialFlow()
        .execute(initRun)
        .execute(deployDB)
        .execute(checkDeployment)
        .execute(generateTSBSData)
        .execute(checkWorkload)
        .execute(loadTSBSData)
        .execute(checkWorkload)
        .execute(generateTSBSQueires)
        .execute(checkWorkload)
        .execute(executeTSBSQueires)
        .execute(checkWorkload)
        .execute(fetchTSBSResultLoad)
        .execute(fetchTSBSResultQuery)
        .execute(systemTimeseries)
        .execute(cleanUpResources)
        .execute(cleanUpRun)
        .build();

    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    runFlow
                )
                .times(this.overallRuns)
                .build()
        )
        //.execute(aggregateEvaluationMetrics)
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);

    return workReport;


  }
}