package de.uulm.omi.evaluation.scenario;

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.ConditionalFlow.Builder.aNewConditionalFlow;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ScalabilityYCSBWrite;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpFailedResources;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchYcsbResult;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.InitRun;
import de.uulm.omi.evaluation.task.LogTask;
import de.uulm.omi.evaluation.task.Plotting;
import de.uulm.omi.evaluation.task.Plotting.PlottingTask;
import de.uulm.omi.evaluation.task.RemoteCommand;
import de.uulm.omi.evaluation.task.TaskState;
import de.uulm.omi.evaluation.task.TaskType;
import de.uulm.omi.evaluation.task.YcsbWorkload;
import de.uulm.omi.utils.YcsbWorkloadType;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkReportPredicate;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;

/**
 * Created by Daniel Seybold on 09.05.2018.
 */
public class ScalabilityYCSBWriteScenario extends EvaluationScenario {

  private final ScalabilityYCSBWrite scalabilityYCSBWrite;
  private final boolean cleanUp;
  private final ClusterContext dbmsClusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final int overallRuns;

  public ScalabilityYCSBWriteScenario(String scenarioType, String cloudType,  String dbmsType, String dbmsConfig, Integer runs,
      ScalabilityYCSBWrite scenarioSpec, boolean cleanUp) {
    super(scenarioType, cloudType, dbmsType , dbmsConfig, runs);

    this.scalabilityYCSBWrite = scenarioSpec;
    this.dbmsClusterContext = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    this.overallRuns = runs.intValue();
    this.cleanUp = cleanUp;
  }

  @Override
  public WorkReport execute() {

    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    InitRun initRun = new InitRun(this.evaluationContext);

    DeployDB deployDB = new DeployDB(this.evaluationContext,
        this.scalabilityYCSBWrite.getDbmsCluster(),
        this.scalabilityYCSBWrite.getDbmsNetwork().getType().name(), this.dbmsClusterContext);

    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext, this.dbmsClusterContext);

    CheckWorkload checkWorkload = new CheckWorkload(this.evaluationContext, this.workloadPhaseContext,
        scalabilityYCSBWrite.getWorkloadInstances());

    FetchYcsbResult fetchYcsbResult = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.LOAD,
        scalabilityYCSBWrite.getWorkloadInstances());


    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    YcsbWorkload ycsbWorkload = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.scalabilityYCSBWrite.getWorkload(), this.scalabilityYCSBWrite.getWorkloadNetwork(), this.workloadPhaseContext,
        YcsbWorkloadType.LOAD, scalabilityYCSBWrite.getWorkloadInstances());

    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, dbmsClusterContext);

    CleanUpFailedResources cleanUpFailedResources = new CleanUpFailedResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    //TODO: finish remote command extension and distinguish openstack/ec2 keypair handling!
    RemoteCommand remoteCommand = new RemoteCommand(this.evaluationContext,this.dbmsClusterContext,this.scalabilityYCSBWrite.getDbmsCluster().getType().name().toLowerCase());


    /**
     * Plot the timeseries of the current run
     */
    Plotting mergeYCSB = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.MERGE_YCSB_LOAD);

    Plotting evaluationTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.EVALUATION_METRICS);

    Plotting systemTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.SYSTEM_METRICS);

    Plotting workloadTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.WORKLOAD_METRICS, scalabilityYCSBWrite
        .getWorkloadInstances());

    Plotting aggregateEvaluationMetrics = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.AGGREGATE_EVALUATION_METRICS);


    SequentialFlow failureAwareFlow;

    failureAwareFlow = aNewSequentialFlow()
        .execute(initRun)
        .execute(deployDB)
        .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.STARTED))
        .then(aNewConditionalFlow()
                .execute(checkDeployment)
                .when(WorkReportPredicate.COMPLETED)
                .then(aNewSequentialFlow()
                    .execute(new LogTask(evaluationContext, TaskType.DEPLOY_DB, TaskState.FINISHED))
                    .execute(ycsbWorkload)
                    .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.STARTED))
                    .execute(checkWorkload)
                    .execute(new LogTask(evaluationContext, TaskType.YCSB_LOAD, TaskState.FINISHED))
                    .execute(fetchYcsbResult)
                    //.execute(remoteCommand)
                    .execute(mergeYCSB)
                    .execute(evaluationTimeseries)
                    .execute(workloadTimeseries)
                    .execute(systemTimeseries)
                    .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.STARTED))
                    .execute(cleanUpResources)
                    .execute(new LogTask(evaluationContext, TaskType.CLEANUP, TaskState.FINISHED))
                    .execute(cleanUpRun)
                    .build())
                .otherwise(
                    aNewSequentialFlow()
                      .execute(cleanUpFailedResources)
                      .execute(cleanUpRun)
                    .build()
                )
                .build())
        .execute(aggregateEvaluationMetrics)
        .build();




    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    failureAwareFlow
                )
                .times(this.overallRuns)
                .build()
        )
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);

    return workReport;

  }
}
