package de.uulm.omi.evaluation.task;

import com.google.common.io.Files;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.dbms.workload.api.DefaultApi;
import de.uulm.omi.dbms.workload.client.ApiException;
import de.uulm.omi.utils.WorkloadApiClientHelper;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 27.04.2018.
 */
public class FetchTSBSResult extends EvaluationTask {



  private final WorkloadPhaseContext workloadPhaseContext;
  private final List<String> workloadInstances;
  private final String RESULT_TYPE;




  public FetchTSBSResult(EvaluationContext evaluationContext, WorkloadPhaseContext workloadPhaseContext, List<String> workloadInstances, String resultType) {
    super(TaskType.FETCH_WORKLOAD, evaluationContext);

    this.workloadPhaseContext = workloadPhaseContext;
    this.workloadInstances = workloadInstances;
    this.RESULT_TYPE = resultType;


  }

  @Override
  public WorkReport call() {

    List<DefaultApi> apiInstances = WorkloadApiClientHelper.getWorkloadClients(workloadInstances);

    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +": ";

    for (DefaultApi apiInstance : apiInstances) {

      try {

        File tmpResultFile = null;
        String dataUri = null;
        if(RESULT_TYPE.equals("load")){
          tmpResultFile = apiInstance.workloadResultGet("tsbs" + evaluationContext.getRunCounter().intValue() + "_" + this.evaluationContext.getScenarioType() + "_load", "TSBS");
          dataUri = this.evaluationContext.getEvluationRun() +  File.separator + EvaluationTask.DATA_FOLDER + File.separator + apiInstances.indexOf(apiInstance) + "_load.txt";
        }else if (RESULT_TYPE.equals("query")){
          tmpResultFile = apiInstance.workloadResultGet("tsbs" + evaluationContext.getRunCounter().intValue() + "_" + this.evaluationContext.getScenarioType(), "TSBS");
          dataUri = this.evaluationContext.getEvluationRun() +  File.separator + EvaluationTask.DATA_FOLDER + File.separator + apiInstances.indexOf(apiInstance) + "_query.txt";
        }else{
          LOGGER.error("Unknown result type: " + this.RESULT_TYPE + "! Supported types are: load, query!" );
          return new DefaultWorkReport(WorkStatus.FAILED);
        }

        LOGGER.debug(loggerPrefix + "Tmp result file location:" + tmpResultFile.getAbsolutePath());

        LOGGER.debug(loggerPrefix + "Storing TSBS workload result file under: " + dataUri);

        File resultFile = new File(dataUri);

        Files.move(tmpResultFile, resultFile);


      } catch (ApiException e) {
        LOGGER.error(loggerPrefix + "Error while fetching workload result!", e);
        return new DefaultWorkReport(WorkStatus.FAILED);

      } catch (IOException e) {

        LOGGER.error(loggerPrefix + "Error while moving workload result!", e);
        return new DefaultWorkReport(WorkStatus.FAILED);
      }
    }


    LOGGER.debug(loggerPrefix + "Successfully fetched workload result and stored to disk!");
    return new DefaultWorkReport(WorkStatus.COMPLETED);
  }


}
