package de.uulm.omi.evaluation.task;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ElasticityStep;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import java.io.File;
import java.util.List;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 02.03.2020.
 */
public class Kaa extends EvaluationTask {

  private final List<ElasticityStep> elasticitySpec;
  ClusterContext dbmsClusterContext;

  public Kaa(EvaluationContext evaluationContext, ClusterContext dbmsClusterContext, List<ElasticityStep> elasticitySpec) {
    super(TaskType.KAA, evaluationContext);

    this.dbmsClusterContext = dbmsClusterContext;
    this.elasticitySpec = elasticitySpec;

  }

  @Override
  public WorkReport call() {

    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +":";


    String specUri =  this.evaluationContext.getEvluationRun()
        + File.separator + EvaluationTask.SPECS_FOLDER
        + File.separator + EvaluationTask.ELASTICITY_SPEC;
    this.writeReport(specUri, GSON.toJson(this.elasticitySpec));

    WorkReport result = null;

    for (ElasticityStep elasticityStep : this.elasticitySpec){


      //execute SCALE Task with optional delay
      result = new Delay(evaluationContext, elasticityStep.getScalingDelay().intValue()).call();
      if (result.getStatus().equals(WorkStatus.FAILED)){
        return result;
      }

      new LogTask(evaluationContext, TaskType.SCALE, TaskState.STARTED).call();
      if (result.getStatus().equals(WorkStatus.FAILED)){
        return result;
      }

      new Scale(this.evaluationContext,this.dbmsClusterContext,elasticityStep).call();
      if (result.getStatus().equals(WorkStatus.FAILED)){
        return result;
      }

      new LogTask(evaluationContext, TaskType.SCALE, TaskState.FINISHED).call();
      if (result.getStatus().equals(WorkStatus.FAILED)){
        return result;
      }


    }


    return result;
  }
}
